package com.zy.bms.controller;

import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.zy.bms.common.ServerResponse;
import com.zy.bms.entity.Group;
import com.zy.bms.pojo.io.GroupIO;
import com.zy.bms.service.IGroupService;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;

/**
 * 项目组 前端控制器
 *
 * @author yangxiaokun
 * @since 2021-06-03
 */
@RestController
@RequestMapping("/bms/group")
public class GroupController {
    @Resource
    private IGroupService groupService;

    /**
     * 分页查询所有项目组
     */
    @PostMapping("listPage.do")
    public ServerResponse listPage(GroupIO io) {
        return ServerResponse.success(groupService.listPage(io));
    }

    /**
     * 添加项目组
     */
    @PostMapping("save.do")
    public ServerResponse save(Group group) {
        try {
            groupService.save(group);
        } catch (Exception e) {
            return ServerResponse.warning("项目组编号重复!");
        }
        return ServerResponse.success();
    }

    /**
     * 禁用项目组
     */
    @PostMapping("forbidden.do")
    public ServerResponse forbidden(String id) {
        groupService.update(new UpdateWrapper<Group>().set("status", 0).eq("id", id));
        return ServerResponse.success();
    }

    /**
     * 启用项目组
     */
    @PostMapping("permit.do")
    public ServerResponse permit(String id) {
        groupService.update(new UpdateWrapper<Group>().set("status", 1).eq("id", id));
        return ServerResponse.success();
    }

    /**
     * 更新项目组信息
     */
    @PostMapping("update.do")
    public ServerResponse update(Group group) {
        return ServerResponse.success(groupService.updateById(group));
    }

}
