package com.zy.bms.controller.ubi;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.zy.bms.common.ServerResponse;
import com.zy.bms.controller.BaseController;
import com.zy.bms.entity.Group;
import com.zy.bms.entity.PrivilegeApplyRecord;
import com.zy.bms.pojo.io.PageIO;
import com.zy.bms.service.IGroupService;
import com.zy.bms.service.IPrivilegeApplyRecordService;
import com.zy.bms.service.IUserPrivilegeService;
import com.zy.bms.service.ubi.IUbiInfoStateService;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

/**
 * 普适型设备
 * 微信小程序 端接口
 *
 * @author yang xiao kun
 * create on 2021/7/16
 */
@RestController
@RequestMapping("/bms/api/ubiapp")
public class UbiWxController extends BaseController {
    @Resource
    private IPrivilegeApplyRecordService privilegeApplyRecordService;
    @Resource
    private IUserPrivilegeService userPrivilegeService;
    @Resource
    private IUbiInfoStateService ubiInfoStateService;
    @Resource
    private IGroupService groupService;

    /**
     * 查看用户拥有权限的设备组列表
     */
    @GetMapping("getOwnGroups.do")
    public ServerResponse getOwnGroups() {
        List<String> groupIds = userPrivilegeService.ownGroupIds(userId());
        return ServerResponse.success(groupService.list(new QueryWrapper<Group>()
                .in("id", groupIds.toArray())));
    }

    /**
     * 申请查看设备组权限
     */
    @PostMapping("apply.do")
    public ServerResponse apply(PrivilegeApplyRecord entity) {
        entity.setUserId(userId());
        return ServerResponse.success(privilegeApplyRecordService.save(entity));
    }

    /**
     * 通过用户ID分页查询该用户拥有权限的设备列表
     */
    @GetMapping("getDeviceList.do")
    public ServerResponse getDeviceList(PageIO io, String group) {
        Set<String> groupIds = new HashSet<>(userPrivilegeService.ownGroupIds(userId()));
        if (!groupIds.contains(group)) return ServerResponse.warning("无权限");
        return ServerResponse.success(ubiInfoStateService.listPageWx(io, group));
    }

    /**
     * 获取设备详情信息
     */
    @GetMapping("getDeviceDetail.do")
    public ServerResponse getDeviceDetail(String openNum) {
        return ServerResponse.success(ubiInfoStateService.getDetailWx(openNum));
    }
}
