package com.zy.bms.handler;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.zy.bms.common.Constant;
import com.zy.bms.entity.lp.LpInfoState;
import com.zy.bms.pojo.dto.MqttDTO;
import com.zy.bms.service.IPostMqttMsgService;
import com.zy.bms.service.lp.ILpInfoStateService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.math.BigDecimal;

/**
 * 路牌 设备消息处理
 *
 * @author yang xiao kun
 * create on 2021/7/26
 */
@Slf4j
@Component
public class LpHandlers {

    @Resource
    private ILpInfoStateService lpInfoStateService;
    @Resource
    private IPostMqttMsgService postMqttMsgService;

    /**
     * 基本信息上传
     */
    public void deviceInfoHandler(String topic, MqttDTO mqttDTO) {
        try {
            String openNum = Constant.splitOpenNum(topic);
            LpInfoState dbDevice = lpInfoStateService.getByOpenNum(openNum);
            LpInfoState device = TransformEntity.tsf2LpInfoState(openNum, mqttDTO);
            //若数据库中不存在设备信息直接插入
            if (dbDevice == null) {
                lpInfoStateService.save(device);
                return;
            }
            BigDecimal wakeInt = BigDecimal.valueOf(mqttDTO.getDouble("sleept")).setScale(2, BigDecimal.ROUND_HALF_UP);
            Integer thresh = mqttDTO.getInt("thresh");
            BigDecimal dbWakeInt = dbDevice.getWakeInt().setScale(2, BigDecimal.ROUND_HALF_UP);
            //如果远程设备的(唤醒时间,阈值)与数据库的数据不一致,则发送消息更新设备的设置数据与数据库保持一致
            if ((dbWakeInt != null && !dbWakeInt.equals(wakeInt)) ||
                    (dbDevice.getThresh() != null && !dbDevice.getThresh().equals(thresh))) {
                postMqttMsgService.updateDeviceInfoLp(openNum, wakeInt, thresh);
            }
            //更新数据库设备状态信息
            lpInfoStateService.update(device, new QueryWrapper<LpInfoState>().eq("open_num", openNum));
        } catch (Exception e) {
            log.error("路牌上传基本信息异常", e);
        }
    }
}
