package com.zy.bms.handler;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.zy.bms.common.Constant;
import com.zy.bms.entity.*;
import com.zy.bms.pojo.dto.MqttDTO;
import com.zy.bms.service.*;
import com.zy.bms.utils.MD5Util;
import com.zy.bms.utils.RabbitMQApi;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;

/**
 * MQTT 消息返回处理类
 *
 * @author yang xiao kun
 * create on 2021/1/19
 */
@Slf4j
@Component
public class MqttCallbackHandler {

    @Resource
    private WebSocketHandler webSocketHandler;
    @Resource
    private IPostMqttMsgService postMqttMsgService;
    @Resource
    private ILogsService logsService;
    @Resource
    private IDeviceBaseService deviceStaticService;
    @Resource
    private UbiHandlers ubiHandlers;

    /**
     * 处理消息
     *
     * @param topic   主题
     * @param payload 消息内容
     */
    public void handle(String topic, String payload) {
        //设备码
        String openNum = Constant.splitOpenNum(topic);
        MqttDTO mqttDTO;
        try {
            //解析消息
            mqttDTO = MqttDTO.parse(payload);
        } catch (Exception e) {
            log.error("mqtt消息格式异常:" + payload, e);
            return;
        }
        // 1. 存储日志 --- 接收
        logsService.saveReceive(topic, payload);
        // 2. 根据报文标识代码 M 处理消息
        switch (mqttDTO.getM()) {
            //设备端注册报文
            case Constant.M.REGISTER: {
                registerHandler(mqttDTO);
                break;
            }
            // 心跳包
            case Constant.M.UBI_HEARTBEAT: {
                ubiHandlers.heartbeatHandler(openNum, mqttDTO);
                break;
            }
            // 获取设备设置信息
            case Constant.M.UBI_SET_INFO: {
                ubiHandlers.deviceSetHandler(openNum, mqttDTO);
                break;
            }
        }
        // 3. 转发消息至网页,忽略注册的报文
        webSocketHandler.massMessage(Constant.splitOpenNum(topic), payload);
    }

    /**
     * 注册设备
     */
    private void registerHandler(MqttDTO mqttDTO) {
        String openNum = mqttDTO.getString("regnum");
        String group = mqttDTO.getString("group");
        // 查询数据库中是否有此设备
        DeviceBase device = deviceStaticService
                .getOne(new QueryWrapper<DeviceBase>()
                        .eq("open_num", openNum));
        // 没有该设备,保存至数据库
        if (device == null) {
            device = new DeviceBase();
            device.setOpenNum(openNum);
            device.setGroupId(group);
            device.setUsername("IMEI" + openNum);
            device.setPassword(MD5Util.encode(openNum));
            device.setSn("SN" + openNum);
            deviceStaticService.save(device);
        }
        // mqtt服务器注册用户
        RabbitMQApi.register(device.getUsername(), device.getPassword());
        // 返回处理结果
        postMqttMsgService.handleRegister(device);
    }
}
