package com.zy.bms.service.impl;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.zy.bms.common.Constant;
import com.zy.bms.entity.Logs;
import com.zy.bms.mapper.LogsMapper;
import com.zy.bms.pojo.dto.MqttDTO;
import com.zy.bms.pojo.io.LogsIO;
import com.zy.bms.pojo.vo.LogsVo;
import com.zy.bms.service.ILogsService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

/**
 * MQTT 收发日志 Service
 *
 * @author chenyi
 * Create on 2020/4/10
 */
@Slf4j
@Service
public class LogsService extends ServiceImpl<LogsMapper, Logs> implements ILogsService {

    @Override
    public void saveReceive(String topic, String payload) {
        saveCommon(topic, payload, 0);
    }

    @Override
    public void saveSend(String topic, String payload) {
        saveCommon(topic, payload, 1);
    }

    @Override
    public IPage<LogsVo> listPage(LogsIO io) {
        Page<LogsVo> page = new Page<>(io.getCurrent(), io.getSize());
        return baseMapper.listPage(page, io);
    }

    /**
     * 保存日志记录
     *
     * @param topic   主题
     * @param content 内容
     * @param type    类型
     */
    private void saveCommon(String topic, String content, Integer type) {
        MqttDTO tmp = MqttDTO.parse(content);
        //创建日志对象
        Logs entity = Logs.builder().topic(topic).type(type).m(tmp.getM()).content(content).build();
        //区分注册报文和普通报文,注册报文取设备随机码
        if (entity.getM().equals(Constant.M.REGISTER) || entity.getM().equals(Constant.M.HANDLE_REGISTER)) {
            entity.setOpenNum(tmp.getString("regnum"));
        } else {
            entity.setOpenNum(Constant.splitOpenNum(topic));
        }
        baseMapper.insert(entity);
    }
}
