package com.zy.bms.service.impl;

import com.zy.bms.common.Constant;
import com.zy.bms.config.mqtt.MqttGateway;
import com.zy.bms.entity.DeviceBase;
import com.zy.bms.entity.lp.LpInfoState;
import com.zy.bms.handler.WebSocketHandler;
import com.zy.bms.pojo.dto.MqttDTO;
import com.zy.bms.pojo.io.set.AlarmIO;
import com.zy.bms.pojo.io.set.NtripIO;
import com.zy.bms.pojo.io.set.SetIO;
import com.zy.bms.pojo.io.set.TcpIO;
import com.zy.bms.service.IPostMqttMsgService;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.HashMap;
import java.util.Map;

/**
 * 向设备发送协议消息 服务类
 *
 * @author yangxiaokun
 * @since 2021-06-03
 */
@Service
public class PostMqttMsgServiceImpl implements IPostMqttMsgService {

    @Resource
    private MqttGateway mqttGateway;
    @Resource
    private LogsService mqttLogService;
    @Resource
    private WebSocketHandler webSocketHandler;

    @Override
    public void set(SetIO io) {
        //主题
        String topic = Constant.TOPIC.DEVICE_CLIENT + io.getOpenNum();
        //消息
        String payload = getMqttDTO(io).json();
        //发送消息
        sendMqttMessage(topic, payload);
    }

    @Override
    public void handleRegister(DeviceBase device) {
        MqttDTO mqttDTO = new MqttDTO(Constant.M.HANDLE_REGISTER);
        mqttDTO.put("regnum", device.getOpenNum())
                .put("password", device.getPassword())
                .put("group", device.getGroupId())
                .put("sn", device.getSn());
        sendMqttMessage(Constant.TOPIC.REGISTER_CLIENT, mqttDTO.json());
    }

    @Override
    public void getDeviceLocalSet(String openNum) {
        MqttDTO mqttDTO = new MqttDTO(Constant.M.UBI_GET_SET_INFO);
        mqttDTO.put("getset", "getset");
        sendMqttMessage(Constant.TOPIC.DEVICE_CLIENT + openNum, mqttDTO.json());
    }

    @Override
    public void updateDeviceInfoLp(String openNum, BigDecimal wakeInt, Integer thresh) {
        MqttDTO mqttDTO = new MqttDTO(Constant.M.LP_UPDATE_DEVICE_INFO);
        mqttDTO.put("setf", "true").put("wakeint", wakeInt).put("thresh", thresh);
        sendMqttMessage(Constant.TOPIC.DEVICE_CLIENT_LP + openNum, mqttDTO.json());
    }

    /**
     * 发送Mqtt 消息
     *
     * @param topic   主题
     * @param payload 内容
     */
    private void sendMqttMessage(String topic, String payload) {
        // 1. 发送 MQTT 协议消息
        mqttGateway.sendMsgToMqtt(payload, topic);
        // 2. 存储消息日志
        mqttLogService.saveSend(Constant.TOPIC.REGISTER_CLIENT, payload);
        // 3. 消息同步转发至webSocket
        webSocketHandler.massMessage(Constant.splitOpenNum(topic), payload);
    }

    /**
     * 组装需要发送的模板消息
     *
     * @param io 设置信息
     */
    private MqttDTO getMqttDTO(SetIO io) {
        MqttDTO result = new MqttDTO(io.getM());
        switch (io.getM()) {
            //普适性 - ntripServer设置
            case Constant.M.UBI_NTRIP_SERVER: {
                NtripIO tmp = (NtripIO) io;
                result.put("en", tmp.getOnOff() == 0 ? "off" : "on").put("host", tmp.getHost())
                        .put("port", tmp.getPort()).put("mountpoint", tmp.getMountPoint())
                        .put("password", tmp.getPassword());
                break;
            }
            //普适性 - ntripClient设置
            case Constant.M.UBI_NTRIP_CLIENT: {
                NtripIO tmp = (NtripIO) io;
                result.put("en", tmp.getOnOff() == 0 ? "off" : "on").put("host", tmp.getHost())
                        .put("port", tmp.getPort()).put("mountpoint", tmp.getMountPoint())
                        .put("password", tmp.getPassword()).put("user", tmp.getUser());
                break;
            }
            //普适性 - tcp设置
            case Constant.M.UBI_TCP: {
                TcpIO tmp = (TcpIO) io;
                result.put("en", tmp.getOnOff() == 0 ? "off" : "on").put("host", tmp.getHost())
                        .put("port", tmp.getPort()).put("type", tmp.getType());
                break;
            }
            //普适性 - 远程控制设备报警
            case Constant.M.UBI_CONTROL_ALARM: {
                AlarmIO tmp = (AlarmIO) io;
                result.put("duration", tmp.getDuration()).put("freq", tmp.getFreq())
                        .put("times", tmp.getTimes());
                break;
            }
        }
        return result;
    }

}
